import logging
from .defaults import ZiaApiBase


class Datacenters(ZiaApiBase):
    def get_all_vips(self):
        path = 'vips?include=all'
        return self._session.get(path)

    def get_all_public_vips(self):
        path = 'vips?include=public'
        return self._session.get(path)

    def get_all_private_vips(self):
        path = 'vips?include=private'
        return self._session.get(path)


LOGGER = logging.getLogger(__name__)



from .defaults import ZiaApiBase


class AdminRoleManagement(ZiaApiBase):
    def list_roles(self):
        """
        Gets a name and ID dictionary of all admin roles
        """
        path = 'adminRoles/lite'
        return self._output(self._session.get(path))

    def list(self):
        """
        Gets a list of admin users
        """
        path = 'adminUsers'
        return self._output(self._session.get(path))

    def create(self, admin):
        """
        Creates an admin or auditor user
        """
        path = 'adminUsers'
        return self._output(self._session.post(path, admin))

    def update(self, user_id, admin):
        """
        Updates an admin or auditor user for the specified ID
        """
        path = 'adminUsers/{}'.format(user_id)
        return self._output(self._session.put(path, admin))

    def delete(self, user_id):
        """
        Deletes an admin or auditor user for the specified ID
        """
        path = 'adminUsers/{}'.format(user_id)
        return self._output(self._session.delete(path))



import logging

from .defaults import ZiaApiBase


class ExemptedUrls(ZiaApiBase):
    def list(self):
        """
        Gets a list of URLs that were exempted from cookie authentication
        """
        path = "authSettings/exemptedUrls"
        return self._output(self._session.get(path))

    def add(self, urls):
        """
        Adds URLs to the cookie authentication exempt list
        """
        path = "authSettings/exemptedUrls?action=ADD_TO_LIST"
        return self._output(self._session.post(path, urls))

    def remove(self, urls):
        """
        Removes URLs from the cookie authentication exempt list
        """
        path = "authSettings/exemptedUrls?action=REMOVE_FROM_LIST"
        return self._output(self._session.post(path, urls))


class AuthSettings(ZiaApiBase):
    def __init__(self, _session, _output_type):
        super().__init__(_session, _output_type)
        self.exempted_urls = ExemptedUrls(self._session, _output_type)


LOGGER = logging.getLogger(__name__)



import logging

from .defaults import ZiaApiBase


class UrlFilteringPolicies(ZiaApiBase):
    def list(self):
        """
        Gets a list of all of URL Filtering Policy rules
        """
        path = 'urlFilteringRules'
        return self._output(self._session.get(path))

    def create(self, rule):
        """
        Adds a URL Filtering Policy rule
        """
        path = 'urlFilteringRules'
        return self._output(self._session.post(path, rule))

    def show(self, rule_id):
        """
        Gets the URL Filtering Policy rule for the specified ID
        """
        path = 'urlFilteringRules/{}'.format(rule_id)
        return self._output(self._session.get(path))

    def update(self, rule_id, rule):
        """
        Updates the URL Filtering Policy rule for the specified ID
        """
        path = 'urlFilteringRules/{}'.format(rule_id)
        return self._output(self._session.put(path, rule))

    def delete(self, rule_id):
        """
        Deletes the URL Filtering Policy rule for the specified ID
        """
        path = 'urlFilteringRules/{}'.format(rule_id)
        return self._output(self._session.delete(path))


LOGGER = logging.getLogger(__name__)



import datetime
import logging

import isodate

from .defaults import ZiaApiBase


class AdminAuditLogs(ZiaApiBase):
    def get(self, _output_type=None):
        """
        Gets the status of a request for an audit log report
        """
        path = 'auditlogEntryReport'
        # status complete
        # download
        return self._output(self._session.get(path), _output_type=_output_type)

    def create(self, start, duration, page=0, page_size=0):
        """
        Creates an audit log report for the specified time period and saves it as a CSV file
        start : iso8601 format (yyyy-mm-ddThh:mm:ss, ex. 2020-10-17T19:13:00)
        duration : iso8601 duration (PdDThHmMsS, ex. P7DT23H59M59S)
        """
        s = isodate.parse_datetime(start)
        e = s + isodate.parse_duration(duration)
        path = 'auditlogEntryReport'
        body = {
            'startTime': int(s.timestamp()*1000),
            'endTime': int(e.timestamp()*1000),
            'page': page,
            'pageSize': page_size
        }
        return self._output(self._session.post(path, body))

    def wait(self, timeout=600):
        """
        Waits for generating report.
        """
        status = {'status': 'not started yet'}
        s = datetime.datetime.now()
        while status is not None and status['status'] != 'COMPLETE':
            status = self.get(_output_type='dict')
            e = datetime.datetime.now()
            if (e - s).seconds > timeout:
                raise RuntimeError('timeout')
        return self._output(status)

    def cancel(self):
        """
        Cancels the request to create an audit log report
        """
        path = 'auditlogEntryReport'
        return self._output(self._session.delete(path))

    def download(self, output):
        """
        Downloads the most recently created audit log report. !!!!!TIMESTAMP IS PDT!!!!!
        output : csv filename
        """
        # timezone が PDT になってる
        self.wait()
        path = 'auditlogEntryReport/download'
        with open(output, 'w') as f:
            f.write(self._session.get(path))
        # return self._output(self._session.get(path))


LOGGER = logging.getLogger(__name__)



import logging

from .defaults import ZiaApiBase


class FilteringRules(ZiaApiBase):
    def list(self):
        """
        Gets all rules in the firewall filtering policy
        """
        path = 'firewallFilteringRules'
        # '/firewallFilteringRules' does not have /lite API
        return self._output(self._session.get(path))

    def show(self, rule_id):
        """
        Gets the firewall filtering rule information for the specified ID
        """
        path = 'firewallFilteringRules/{}'.format(rule_id)
        return self._output(self._session.get(path))


class IpDestinationGroups(ZiaApiBase):
    def list(self, summary=False):
        """
        Gets a list of all IP destination groups
        --summary : Gets a name and ID dictionary of all IP destination groups
        """
        path = 'ipDestinationGroups'
        if summary:
            path += '/lite'
        return self._output(self._session.get(path))

    def show(self, ip_group_id):
        """
        Gets the IP destination group information for the specified ID
        """
        path = 'ipDestinationGroups/{}'.format(ip_group_id)
        return self._output(self._session.get(path))


class IpSourceGroups(ZiaApiBase):
    def list(self, summary=False):
        """
        Gets a list of all IP source groups
        --summary : Gets a name and ID dictionary of all IP source groups
        """
        path = 'ipSourceGroups'
        if summary:
            path += '/lite'
        return self._output(self._session.get(path))

    def show(self, ip_group_id):
        """
        Gets the IP source group information for the specified ID
        """
        path = 'ipSourceGroups/{}'.format(ip_group_id)
        return self._output(self._session.get(path))


class NetworkApplicationGroups(ZiaApiBase):
    def list(self, summary=False):
        """
        Gets a list of all network application groups
        --summary : Gets a name and ID dictionary of all network application groups
        """
        path = 'networkApplicationGroups'
        if summary:
            path += '/lite'
        return self._output(self._session.get(path))

    def show(self, group_id):
        """
        Gets the network application group information for the specified ID
        """
        path = 'networkApplicationGroups/{}'.format(group_id)
        return self._output(self._session.get(path))


class NetworkApplications(ZiaApiBase):
    def list(self, summary=False):
        """
        Gets a list of all predefined network applications
        """
        path = 'networkApplications'
        # '/networkAplications' does not have /lite API
        return self._output(self._session.get(path))

    def show(self, app_id):
        """
        Gets the network application information for the specified ID
        """
        path = 'networkApplications/{}'.format(app_id)
        return self._output(self._session.get(path))


class NetworkServiceGroups(ZiaApiBase):
    def list(self, summary=False):
        """
        Gets a list of all network service groups
        --summary : Gets a name and ID dictionary of all network service groups
        """
        path = 'networkServiceGroups'
        if summary:
            path += '/lite'
        return self._output(self._session.get(path))

    def show(self, service_group_id):
        """
        Gets the network service group information for the specified ID
        """
        path = 'networkServiceGroups/{}'.format(service_group_id)
        return self._output(self._session.get(path))


class NetworkServices(ZiaApiBase):
    def list(self, summary=False):
        """
        Gets a list of all network services
        --summary : Gets a name and ID dictionary of all network services
        """
        path = 'networkServices'
        if summary:
            path += '/lite'
        return self._output(self._session.get(path))

    def show(self, service_id):
        """
        Gets the network service for the specified ID
        """
        path = 'networkServices/{}'.format(service_id)
        return self._output(self._session.get(path))


class TimeWindows(ZiaApiBase):
    def list(self, summary=False):
        """
        Gets a list of time intervals used by the firewall filtering policy
        --summary : Gets a name and ID dictionary of time intervals used by the firewall filtering policy
        """
        path = 'timeWindows'
        if summary:
            path += '/lite'
        return self._output(self._session.get(path))


class Firewall(ZiaApiBase):
    def __init__(self, _session, _output_type):
        super().__init__(_session, _output_type)
        self.rules = FilteringRules(self._session, _output_type)
        self.destination = IpDestinationGroups(self._session, _output_type)
        self.source = IpSourceGroups(self._session, _output_type)
        self.application_groups = NetworkApplicationGroups(
            self._session, _output_type)
        self.applications = NetworkApplications(self._session, _output_type)
        self.service_groups = NetworkServiceGroups(self._session, _output_type)
        self.services = NetworkServices(self._session, _output_type)
        self.timewindows = TimeWindows(self._session, _output_type)


LOGGER = logging.getLogger(__name__)



import logging

import fire

from .defaults import load_config, RequestError
from zia import ZscalerInternetAccess


def main():
    load_config()
    LOGGER = logging.getLogger(__name__)
    LOGGER.setLevel(logging.DEBUG)
    try:
        z = ZscalerInternetAccess()
        z.authenticate()
        fire.Fire(z)
    except RequestError as exc:
        LOGGER.error(exc)
    return 0


if __name__ == "__main__":
    main()



from .session import Session

from .activation import Activation
from .admin_audit_logs import AdminAuditLogs
from .admin_role_management import AdminRoleManagement
from .cloud_sandbox_report import CloudSandboxReport
from .firewall import Firewall
# from .datacenters import Datacenters
# from .gre import Gre
from .locations import Locations
from .security import Security
# from .sandbox import Sandbox
from .ssl_inspection_settings import SslSettings
from .user_management import Departments, Groups, Users
from .traffic_forwarding import VpnCredentials
from .url_filtering_policies import UrlFilteringPolicies
from .url_categories import UrlCategories
from .user_authentication_settings import AuthSettings


class ZscalerInternetAccess(object):
    def __init__(self, profile='default'):
        self._session = Session(profile=profile)
        self.activation = Activation(self._session, 'str')
        self.admin_audit_logs = AdminAuditLogs(self._session, 'str')
        self.admin_role_management = AdminRoleManagement(self._session, 'str')
        self.sandbox = CloudSandboxReport(self._session, 'str')
        self.firewall = Firewall(self._session, 'str')
        self.locations = Locations(self._session, 'str')
        self.security = Security(self._session, 'str')
        self.ssl = SslSettings(self._session, 'str')
        self.department = Departments(self._session, 'str')
        self.group = Groups(self._session, 'str')
        self.user = Users(self._session, 'str')
        self.vpn = VpnCredentials(self._session, 'str')
        self.policies = UrlFilteringPolicies(self._session, 'str')
        self.categories = UrlCategories(self._session, 'str')
        self.auth_settings = AuthSettings(self._session, 'str')
        # self.datacenters = Datacenters(self._session, 'str')
        # self.gre = Gre(self._session, 'str')

    def authenticate(self):
        self._session.authenticate()



import logging

from .defaults import ZiaApiBase


class Activation(ZiaApiBase):
    def get_status(self):
        """
        Gets the activation status for a configuration change
        """
        path = 'status'
        return self._output(self._session.get(path))

    def activate(self):
        """
        Activates configuration changes
        """
        path = 'status/activate'
        body = {}
        return self._output(self._session.post(path, body))


LOGGER = logging.getLogger(__name__)



import logging

from .defaults import ZiaApiBase


class Departments(ZiaApiBase):
    def list(self):
        """
        Gets a list of departments
        """
        path = "departments"
        return self._output(self._session.get(path))

    def show(self, department_id):
        """
        Gets the department for the specified ID
        """
        path = "departments/{}".format(department_id)
        return self._output(self._session.get(path))


class Groups(ZiaApiBase):
    def list(self):
        """
        Gets a list of groups
        """
        path = "groups"
        return self._output(self._session.get(path))

    def show(self, group_id):
        """
        Gets the group for the specified ID
        """
        path = "groups/{}".format(group_id)
        return self._output(self._session.get(path))


class Users(ZiaApiBase):
    def list(self):
        """
        Gets a list of all users and allows user filtering by name, department, or group
        """
        path = "users"
        return self._output(self._session.get(path))

    def show(self, user_id):
        """
        Gets the user information for the specified ID
        """
        path = "users/{}".format(user_id)
        return self._output(self._session.get(path))

    def create(self, user):
        """
        Adds a new user
        """
        path = "users"
        return self._output(self._session.post(path, user))

    def update(self, user_id, user):
        """
        Updates the user information for the specified ID
        """
        path = "users/{}".format(user_id)
        return self._output(self._session.put(path, user))

    def delete(self, user_object):
        """
        user_id : Deletes the user for the specified ID
        user_ids : Bulk delete users up to a maximum of 500 users per request
        """
        t = type(user_object)
        if t is int or t is str:
            path = "users/{}".format(user_object)
            return self._output(self._session.delete(path))
        elif t is dict:
            path = "users/bulkDelete"
            return self._output(self._session.post(path, user_object))
        raise RuntimeError('unknown user_object type {}'.format(t.__name__))


LOGGER = logging.getLogger(__name__)



import logging
from .defaults import ZiaApiBase


class CertificateSigningRequest(ZiaApiBase):
    def download(self, csr):
        """
        Downloads a Certificate Signing Request (CSR)
        """
        path = "sslSettings/downloadcsr"
        return self._output(self._session.get(path, csr))

    def generate(self, csr):
        """
        Generates a Certificate Signing Request (CSR)
        """
        path = "sslSettings/generatecsr"
        return self._output(self._session.post(path, csr))


class Certificate(ZiaApiBase):
    def show(self):
        """
        Shows information about the signed intermediate root certificate
        """
        path = "sslSettings/showcert"
        return self._output(self._session.get(path))

    def upload(self):
        """
        Uploads a signed intermediate root certificate for clients that use iframe-based uploads whose content type is text/plain
        """
        path = "sslSettings/uploadcert/text"
        return self._output(self._session.post(path))


class CertificateChain(ZiaApiBase):
    def upload(self):
        """
        Uploads the Intermediate Certificate Chain (PEM) for clients that use iframe-based uploads whose content type is text/plain
        """
        path = "sslSettings/uploadcertchain/text"
        return self._output(self._session.post(path))

    def delete(self):
        """
        Deletes the intermediate certificate chain
        """
        path = "sslSettings/certchain"
        return self._output(self._session.delete(path))


class ExemptedUrls(ZiaApiBase):
    def get(self):
        """
        Gets a list of URLs that were exempted from SSL Inpection and policy evaluation
        """
        path = "sslSettings/exemptedUrls"
        return self._output(self._session.get(path))

    def add(self):
        """
        Adds URLs to the exempted from SSL Inspection and policy evaluation list or removes URLs from the list
        """
        path = "sslSettings/exemptedUrls"
        return self._output(self._session.post(path))


class SslSettings(ZiaApiBase):
    def __init__(self, _session, _output_type):
        super().__init__(_session, _output_type)
        self.csr = CertificateSigningRequest(self._session, _output_type)
        self.certificate = Certificate(self._session, _output_type)
        self.chain = CertificateChain(self._session, _output_type)
        self.exempted_urls = ExemptedUrls(self._session, _output_type)


LOGGER = logging.getLogger(__name__)



import random
import string
import logging

from .defaults import ZiaApiBase


class VpnCredentials(ZiaApiBase):
    MAX_PSK_LEN = 64

    def _randomize_psk(self):
        psk = ''.join(random.choices(
            string.ascii_letters + string.digits, k=self.MAX_PSK_LEN))
        LOGGER.debug("RANDOM PSK: {} (PSK Length: {})".format(
            psk,
            len(psk)
        ))
        return psk

    def list(self):
        path = 'vpnCredentials'
        return self._output(self._session.get(path))

    def create(self, credential):
        path = 'vpnCredentials'
        if 'preSharedKey' not in credential:
            credential['preSharedKey'] = self._randomize_psk()
        return self._output(self._session.post(path, credential))

    def show(self, vpn_id):
        path = 'vpnCredentials/{}'.format(vpn_id)
        return self._output(self._session.get(path))

    def update(self, vpn_id, credential):
        path = 'vpnCredentials/{}'.format(vpn_id)
        if 'preSharedKey' not in credential:
            credential['preSharedKey'] = self._randomize_psk()
        return self._output(self._session.put(path, credential))

    def delete(self, vpn_id):
        path = 'vpnCredentials/{}'.format(vpn_id)
        return self._output(self._session.delete(path))


LOGGER = logging.getLogger(__name__)



from .defaults import load_config, get_config, RequestError, SessionTimeoutError, ZiaApiBase
from .session import Session

from .activation import Activation
from .admin_audit_logs import AdminAuditLogs
from .admin_role_management import AdminRoleManagement
from .cloud_sandbox_report import CloudSandboxReport
from .firewall import Firewall
from .datacenters import Datacenters
from .gre import Gre
from .locations import Locations
from .security import Security
from .sandbox import Sandbox
from .ssl_inspection_settings import SslSettings
from .user_management import Departments, Groups, Users
from .traffic_forwarding import VpnCredentials
from .url_filtering_policies import UrlFilteringPolicies
from .url_categories import UrlCategories
from .user_authentication_settings import AuthSettings

from .zscaler_internet_access import ZscalerInternetAccess



import logging

from .defaults import ZiaApiBase


class UrlCategories(ZiaApiBase):
    def list(self, custom_only=False, summary=False):
        """
        Gets information about all or custom URL categories
        --summary : Gets a lightweight key-value list of all or custom URL categories
        """
        path = 'urlCategories'
        if summary:
            path += '/lite'
        if custom_only:
            path += '?customOnly=true'
        return self._output(self._session.get(path))

    def create(self, category):
        """
        Adds a new custom URL category
        """
        path = 'urlCategories'
        return self._output(self._session.post(path, category))

    def get_quota(self):
        """
        Gets the URL quota information for your organization
        """
        path = 'urlCategories/urlQuota'
        return self._output(self._session.get(path))

    def get(self, category_id):
        """
        Gets the URL category information for the specified ID
        """
        path = 'urlCategories/{}'.format(category_id)
        return self._output(self._session.get(path))

    def update(self, category_id, category):
        """
        Updates the URL category for the specified ID
        """
        path = 'urlCategories/{}'.format(category_id)
        return self._output(self._session.put(path, category))

    def delete(self, category_id):
        """
        Deletes the custom URL category for the specified ID
        """
        path = 'urlCategories/{}'.format(category_id)
        return self._output(self._session.delete(path))

    def lookup(self, urls):
        """
        Look up the categorization of the given set of URLs, e.g., ['abc.com', 'xyz.com']
        """
        path = 'urlLookup'
        return self._output(self._session.post(path, urls))


LOGGER = logging.getLogger(__name__)



import logging

from .defaults import ZiaApiBase


class SubLocations(ZiaApiBase):
    def list(self, location_id):
        """
        Gets the sub-location information for the location with the specified ID
        """
        path = 'locations/{}/sublocations'.format(location_id)
        return self._output(self._session.get(path))

    def create(self, sublocation):
        """
        Adds new sub-locations for the specified location_id
        """
        path = 'locations'
        if 'parentId' not in sublocation:
            raise RuntimeError('parentId required')
        return self._output(self._session.post(path, sublocation))


class Locations(ZiaApiBase):
    def __init__(self, _session, _output_type):
        super().__init__(_session, _output_type)
        self.sublocations = SubLocations(self._session, _output_type)

    def list(self, summary=False):
        """
        Gets information on locations
        --summary : Gets a name and ID dictionary of locations
        """
        path = 'locations'
        if summary:
            path += '/lite'
        return self._output(self._session.get(path))

    def create(self, location):
        """
        Adds new locations
        """
        path = 'locations'
        return self._output(self._session.post(path, location))

    def show(self, location_id):
        """
        Gets the location information for the specified ID
        """
        if not location_id:
            return "Location Requried"
        path = 'locations/{}'.format(location_id)
        return self._output(self._session.get(path))

    def update(self, location_id, location):
        """
        Updates the location and sub-location information for the specified ID
        """
        path = 'locations/{}'.format(location_id)
        return self._output(self._session.put(path, location))

    def delete(self, location_object):
        """
        location_id : Deletes the location or sub-location for the specified ID
        location_ids : Bulk delete locations up to a maximum of 100 locations per request
        """
        t = type(location_object)
        if t is int or t is str:
            path = 'locations/{}'.format(location_object)
            return self._output(self._session.delete(path))
        elif t is dict:
            path = 'locations/bulkDelete'
            return self._output(self._session.post(path, location_object))
        raise RuntimeError(
            'unknown location_object type {}'.format(t.__name__))


LOGGER = logging.getLogger(__name__)



import logging

from zia import ZiaApiBase


class WhiteList(ZiaApiBase):
    def get(self):
        """
        Gets a list of white-listed URLs
        """
        path = 'security'
        return self._output(self._session.get(path))

    def update(self, urls):
        """
        Updates the list of white-listed URLs
        """
        path = 'security'
        return self._output(self._session.put(path, urls))


class BlackList(ZiaApiBase):
    def get(self):
        """
        Gets a list of black-listed URLs
        """
        path = 'security/advanced'
        return self._output(self._session.get(path))

    def update(self, urls):
        """
        Updates the list of black-listed URLs
        """
        path = 'security/advanced'
        return self._output(self._session.put(path, urls))

    def add(self, urls):
        """
        Adds URLs to the black list
        """
        path = 'security/advanced/blacklistUrls?action=ADD_TO_LIST'
        return self._output(self._session.post(path, urls))

    def remove(self, urls):
        """
        Removes URLs from the black list
        """
        path = 'security/advanced/blacklistUrls?action=REMOVE_FROM_LIST'
        return self._output(self._session.post(path, urls))


class Security(ZiaApiBase):
    def __init__(self, _session, _output_type):
        super().__init__(_session, _output_type)
        self.whitelist = WhiteList(self._session, _output_type)
        self.blacklist = BlackList(self._session, _output_type)


LOGGER = logging.getLogger(__name__)



import logging
from .defaults import ZiaApiBase


class CloudSandboxReport(ZiaApiBase):
    def get_quota(self):
        """
        Gets the Sandbox Report API quota information for your organization
        """
        path = 'sandbox/report/quota'
        return self._output(self._session.get(path))

    def get_report(self, md5hash, details="summary"):
        """
        Gets a full (i.e., complete) or summary detail report for an MD5 hash of a file that was analyzed by Sandbox
        """
        if details not in ['summary', 'full']:
            raise RuntimeError('details must be summary or full')
        path = 'sandbox/report/{}?details={}'.format(md5hash, details)
        return self._output(self._session.get(path))


LOGGER = logging.getLogger(__name__)
